/*
 * uFR.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>
#include <stdbool.h>
#include <string.h>
#include <uFCoder.h>
#include "ini.h"
#include "uFR.h"

//------------------------------------------------------------------------------
void HexStrCopyAndPurify(char *out_buff, sz_ptr in_hex_str) {
	size_t str_len = strlen(in_hex_str);
	size_t j = 0;

	for (size_t i = 0; i < str_len; i++) {
		if (((in_hex_str[i] >= '0') && (in_hex_str[i] <= '9')) || ((in_hex_str[i] >= 'A') && (in_hex_str[i] <= 'F'))
				|| ((in_hex_str[i] >= 'a') && (in_hex_str[i] <= 'f'))) {
			out_buff[j++] = in_hex_str[i];
		}
	}
	out_buff[j] = '\0';
}
//------------------------------------------------------------------------------
size_t hex2bin_introvert(uint8_t *buff) {
	size_t str_len = strlen((char *) buff);
	uint8_t *src = buff, *dst = buff;
	char s_tmp[3];

	s_tmp[2] = '\0';

	for (size_t i = 0; i < str_len; i++) {
		s_tmp[0] = (char) *src++;
		s_tmp[1] = (char) *src++;

		*dst++ = strtoul(s_tmp, NULL, 16);
	}
	return str_len / 2;
}
//------------------------------------------------------------------------------
UFR_STATUS ApduCommand(sz_ptr apdu, uint8_t *response, size_t *response_len, uint8_t *sw) {
	size_t apdu_len = strlen(apdu);
	uint8_t *buff, *out_data, lc, send_le = 0;
	uint32_t le;
	UFR_STATUS status;

	if (apdu_len - 1 > MAX_ALLOC_MEM)
		return UFR_BUFFER_OVERFLOW;

	buff = malloc(apdu_len);

	HexStrCopyAndPurify((char *) buff, apdu);
	apdu_len = strlen((char *) buff);
	if (apdu_len % 2) {
		free(buff);
		return UFR_PARAMETERS_ERROR;
	}

	apdu_len = hex2bin_introvert(buff);
	if (apdu_len < 4) {
		free(buff);
		return UFR_PARAMETERS_ERROR;
	}

	if (apdu_len == 4) {
		out_data = NULL;
		lc = 0;
		le = 0;
		send_le = 0;
	} else if (apdu_len == 5) {
		out_data = NULL;
		lc = 0;
		le = buff[4];
		send_le = 1;
	} else { // 6 and more bytes...
		lc = buff[4];
		if (!lc) {
			free(buff);
			return UFR_PARAMETERS_ERROR;
		}

		out_data = &buff[5];

		le = apdu_len - lc - 5;
		if (le > 1) {
			free(buff);
			return UFR_PARAMETERS_ERROR;
		} else if (le) {
			le = buff[apdu_len - 1];
			send_le = 1;
		} else {
			send_le = 0;
		}
	}

	status = uFR_APDU_Transceive(buff[0], buff[1], buff[2], buff[3], out_data, lc, response, *response_len, &le, send_le, sw);

	*response_len = le >= 2 ? le - 2 : 0;
	free(buff);
	return status;
}
//------------------------------------------------------------------------------
bool CheckDependencies(void) {
#if defined(EXIT_ON_WRONG_FW_DEPENDENCY) || defined(EXIT_ON_WRONG_LIB_DEPENDENCY)
	uint8_t version_major, version_minor, build;
	bool wrong_version = false;
#endif
	UFR_STATUS status;

#ifdef EXIT_ON_WRONG_LIB_DEPENDENCY
#endif
#ifdef EXIT_ON_WRONG_FW_DEPENDENCY
	wrong_version = false;
	status = GetReaderFirmwareVersion(&version_major, &version_minor);
	if (status != UFR_OK) {
		printf(" Error while checking device version, status is: 0x%08X\n", status);
		return false;
	}
	status = GetBuildNumber(&build);
	if (status != UFR_OK) {
		printf(" Error while checking device version, status is: 0x%08X\n", status);
		return false;
	}
	if (version_major < MIN_DEPEND_FW_VER_MAJOR) {
		wrong_version = true;
	} else if (version_major == MIN_DEPEND_FW_VER_MAJOR && version_minor < MIN_DEPEND_FW_VER_MINOR) {
		wrong_version = true;
	} else if (version_minor == MIN_DEPEND_FW_VER_MINOR && build < MIN_DEPEND_FW_VER_BUILD) {
		wrong_version = true;
	}

	if (wrong_version) {
		printf("Wrong uFR NFC reader firmware version (%d.%d.%d).\n"
				"Please update uFR firmware to at last %d.%d.%d version.\n", version_major, version_minor, build,
		MIN_DEPEND_FW_VER_MAJOR, MIN_DEPEND_FW_VER_MINOR, MIN_DEPEND_FW_VER_BUILD);
		return false;
	}
#endif
	return true;
}
//------------------------------------------------------------------------------
sz_ptr GetDlTypeName(uint8_t dl_type_code) {

	switch (dl_type_code) {
	case DL_MIFARE_ULTRALIGHT:
		return "DL_MIFARE_ULTRALIGHT";
	case DL_MIFARE_ULTRALIGHT_EV1_11:
		return "DL_MIFARE_ULTRALIGHT_EV1_11";
	case DL_MIFARE_ULTRALIGHT_EV1_21:
		return "DL_MIFARE_ULTRALIGHT_EV1_21";
	case DL_MIFARE_ULTRALIGHT_C:
		return "DL_MIFARE_ULTRALIGHT_C";
	case DL_NTAG_203:
		return "DL_NTAG_203";
	case DL_NTAG_210:
		return "DL_NTAG_210";
	case DL_NTAG_212:
		return "DL_NTAG_212";
	case DL_NTAG_213:
		return "DL_NTAG_213";
	case DL_NTAG_215:
		return "DL_NTAG_215";
	case DL_NTAG_216:
		return "DL_NTAG_216";
	case DL_MIKRON_MIK640D:
		return "DL_MIKRON_MIK640D";
	case DL_MIFARE_MINI:
		return "DL_MIFARE_MINI";
	case DL_MIFARE_CLASSIC_1K:
		return "DL_MIFARE_CLASSIC_1K";
	case DL_MIFARE_CLASSIC_4K:
		return "DL_MIFARE_CLASSIC_4K";
	case DL_MIFARE_PLUS_S_2K:
		return "DL_MIFARE_PLUS_S_2K";
	case DL_MIFARE_PLUS_S_4K:
		return "DL_MIFARE_PLUS_S_4K";
	case DL_MIFARE_PLUS_X_2K:
		return "DL_MIFARE_PLUS_X_2K";
	case DL_MIFARE_PLUS_X_4K:
		return "DL_MIFARE_PLUS_X_4K";
	case DL_MIFARE_DESFIRE:
		return "DL_MIFARE_DESFIRE";
	case DL_MIFARE_DESFIRE_EV1_2K:
		return "DL_MIFARE_DESFIRE_EV1_2K";
	case DL_MIFARE_DESFIRE_EV1_4K:
		return "DL_MIFARE_DESFIRE_EV1_4K";
	case DL_MIFARE_DESFIRE_EV1_8K:
		return "DL_MIFARE_DESFIRE_EV1_8K";
	case DL_UNKNOWN_ISO_14443_4:
		return "DL_GENERIC_ISO_14443_4";
	case DL_IMEI_UID:
		return "DL_IMEI_UID";
	}
	return "UNSUPPORTED CARD";
}
//------------------------------------------------------------------------------
